@@ -32,8 +32,8 @@ test_that("Interrogating with an agent yields the correct results", {
   schema <- 
     col_schema(
       date_time = c("POSIXct", "POSIXt"), date = "Date", 
-      a = "integer", b = "character", c = "numeric", d = "numeric", 
-      e = "logical", f = "character"
+      a = "integer", b = "character", c = "numeric",
+      d = "numeric", e = "logical", f = "character"
     )
   
   validation <-
@@ -57,6 +57,18 @@ test_that("Interrogating with an agent yields the correct results", {
   # Expect a single row in `validation$validation_set`
   expect_equivalent(nrow(validation$validation_set), 1)
   
+  # Expect an error if a `col_schema` object isn't provided to `schema`
+  expect_error(
+    create_agent(tbl = small_table) %>%
+      col_schema_match(
+        schema = list(
+          date_time = c("POSIXct", "POSIXt"), date = "Date", 
+          a = "integer", b = "character", c = "numeric",
+          d = "numeric",  e = "logical", f = "character"
+        )
+      )
+  )
+  
   # Use the `col_exists()` function to create
   # a validation step, then, `interrogate()`
   # (expecting a failed interrogation)
@@ -248,7 +260,7 @@ test_that("Interrogating for valid row values", {
     col_vals_between(
       columns = vars(d),
       left = 0, right = 5000,
-      preconditions = ~ tbl %>% dplyr::filter(date > "2016-01-04")
+      preconditions = ~ . %>% dplyr::filter(date > "2016-01-04")
     ) %>%
     interrogate()
   
@@ -349,7 +361,7 @@ test_that("Interrogating for valid row values", {
     col_vals_not_between(
       columns = vars(d),
       left = 500, right = 1000,
-      preconditions = ~ tbl %>% dplyr::filter(date > "2016-01-04")
+      preconditions = ~ . %>% dplyr::filter(date > "2016-01-04")
     ) %>%
     interrogate()
   
@@ -395,7 +407,7 @@ test_that("Interrogating for valid row values", {
     col_vals_equal(
       columns = vars(d),
       value = 283.94,
-      preconditions = ~ tbl %>% dplyr::filter(date > "2016-01-04")
+      preconditions = ~ . %>% dplyr::filter(date > "2016-01-04")
     ) %>%
     interrogate()
   
@@ -464,7 +476,7 @@ test_that("Interrogating for valid row values", {
     col_vals_not_equal(
       columns = vars(d),
       value = 283.94,
-      preconditions = ~ tbl %>% dplyr::filter(date > "2016-01-04")
+      preconditions = ~ . %>% dplyr::filter(date > "2016-01-04")
     ) %>%
     interrogate()
   
@@ -511,7 +523,7 @@ test_that("Interrogating for valid row values", {
       create_agent(tbl = small_table) %>%
       col_vals_gt(
         columns = vars(date_time), value = vars(date),
-        preconditions = ~ tbl %>% dplyr::mutate(date = lubridate::as_datetime(date))) %>%
+        preconditions = ~ . %>% dplyr::mutate(date = lubridate::as_datetime(date))) %>%
       interrogate()
   )
   
@@ -558,7 +570,7 @@ test_that("Interrogating for valid row values", {
   validation <-
     create_agent(tbl = small_table) %>%
     rows_distinct(
-      preconditions = ~ tbl %>% dplyr::filter(date != "2016-01-20")
+      preconditions = ~ . %>% dplyr::filter(date != "2016-01-20")
     ) %>%
     interrogate()
   
@@ -690,7 +702,7 @@ test_that("Interrogating for valid row values", {
     create_agent(tbl = small_table) %>%
     col_vals_not_null(
       columns = vars(c),
-      preconditions = ~ tbl %>%
+      preconditions = ~ . %>%
         dplyr::filter(date > "2016-01-06" & date < "2016-01-30")
     ) %>%
     interrogate()
@@ -736,7 +748,7 @@ test_that("Interrogating for valid row values", {
     create_agent(tbl = small_table) %>%
     col_vals_null(
       columns = vars(c),
-      preconditions = ~ tbl %>%
+      preconditions = ~ . %>%
         dplyr::filter(date == '2016-01-06' | date == '2016-01-30')
     ) %>%
     interrogate()
@@ -786,7 +798,7 @@ test_that("Interrogating for valid row values", {
     col_vals_regex(
       columns = vars(f),
       regex = "[a-z]{3}",
-      preconditions = ~ tbl %>% dplyr::filter(f != "high")
+      preconditions = ~ . %>% dplyr::filter(f != "high")
     ) %>%
     interrogate()
   
